using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace AvatarSample {

	class Actor {
		readonly GraphicsDevice graphicsDevice;
		AvatarDescription description;
		AvatarAnimation animation;
		public AvatarRenderer Renderer { get; protected set; }
		public AvatarExpression Expression;
		public bool IsLoaded {
			get {
				if (Renderer == null) return false;
				return Renderer.IsLoaded;
			}
		}
		public Actor(GraphicsDevice graphicsDevice) {
			this.graphicsDevice = graphicsDevice;
		}
		public void CreateAtRandom() {
			animation = new AvatarAnimation(AvatarAnimationPreset.Wave);
			description = AvatarDescription.CreateRandom();
			Renderer = new AvatarRenderer(description);
		}
		public void CreateFrom(AvatarDescription description) {
			animation = new AvatarAnimation(AvatarAnimationPreset.Wave);
			this.description = description;
			Renderer = new AvatarRenderer(description);
		}
		public void Update(GameTime gameTime) {
			if (animation != null)
				animation.Update(gameTime.ElapsedGameTime, true);
		}
		public void Draw(GameTime gameTime) {
			if (Renderer != null) {
				Renderer.LightDirection = Vector3.Normalize(new Vector3(1, -2, 3));
				Renderer.World = Matrix.CreateRotationY(RotationY) * Matrix.CreateRotationX(RotationX)
					* World * Camera.World;
				Renderer.View = Camera.View;
				Renderer.Projection = Camera.Projection;
				Renderer.Draw(animation.BoneTransforms, Expression);
			}
		}
		public Camera Camera;
		public float RotationX;
		public float RotationY;
		public Matrix World = Matrix.Identity;
	}

	public class Camera {
		public Matrix World;
		public Matrix View;
		public Matrix Projection;
	}

	public partial class AvatarCircleGame : Microsoft.Xna.Framework.Game {
		readonly GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		readonly List<Actor> actors = new List<Actor>();
		readonly Camera camera = new Camera();
		int targetIndex;

		public AvatarCircleGame() {
			graphics = new GraphicsDeviceManager(this);
			graphics.PreferMultiSampling = true;
			graphics.PreferredBackBufferWidth = 1280;
			graphics.PreferredBackBufferHeight = 720;
			Content.RootDirectory = "Content";
			// AvatarRenderer̂߂ɕKv
			Components.Add(new GamerServicesComponent(this));
		}

		protected override void LoadContent() {
			spriteBatch = new SpriteBatch(GraphicsDevice);
			var count = 32;
			var radius = 5;
			for (int i = 0; i < count; i++) {
				var actor = new Actor(GraphicsDevice);
				actor.Camera = camera;
				var angle = MathHelper.TwoPi / count * i;
				actor.World = Matrix.CreateTranslation(0, 0, -radius)
				* Matrix.CreateRotationY(-angle);
				actors.Add(actor);
			}
			actors[0].CreateAtRandom();
		}

		void HandleInput() {
			var state = GamePad.GetState(PlayerIndex.One);
			if (state.Buttons.Back == ButtonState.Pressed)
				this.Exit();
			actors[0].RotationX = state.ThumbSticks.Left.Y * MathHelper.Pi;
			actors[0].RotationY = state.ThumbSticks.Left.X * MathHelper.Pi;

			if (state.IsButtonDown(Buttons.Y)) {
				if (actors[0].IsLoaded && actors[targetIndex].IsLoaded) {
					if (++targetIndex >= actors.Count) targetIndex = 0;
					actors[targetIndex].CreateAtRandom();
				}
			}
			if (state.IsButtonDown(Buttons.X)) {
				if (actors[0].IsLoaded && actors[targetIndex].IsLoaded) {
					if (Gamer.SignedInGamers.Count > 0)
						actors[0].CreateFrom(Gamer.SignedInGamers[0].Avatar);
				}
			}
			if (state.IsButtonDown(Buttons.A)) {
				var expression = new AvatarExpression();
				expression.Mouth = AvatarMouth.Happy;
				expression.LeftEye = AvatarEye.Happy;
				expression.RightEye = AvatarEye.Happy;
				expression.LeftEyebrow = AvatarEyebrow.Neutral;
				expression.RightEyebrow = AvatarEyebrow.Neutral;

				actors[0].Expression = expression;
			} else {
				actors[0].Expression = new AvatarExpression();
			}
		}

		protected override void Update(GameTime gameTime) {
			HandleInput();
			foreach (var actor in actors) {
				actor.Update(gameTime);
			}
			camera.World =
				Matrix.CreateRotationY((float)(gameTime.TotalGameTime.TotalSeconds * 0.1f))
				* Matrix.CreateTranslation(0, 0, 5);
			camera.View = Matrix.CreateLookAt(new Vector3(0, 1, -3), new Vector3(0, 1, 0), Vector3.Up); ;
			camera.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
				GraphicsDevice.Viewport.AspectRatio, 1, 1000);

			base.Update(gameTime);
		}

		protected override void Draw(GameTime gameTime) {
			GraphicsDevice.Clear(Color.CornflowerBlue);
			foreach (var actor in actors) {
				actor.Draw(gameTime);
			}
			base.Draw(gameTime);
		}
	}
}
